//////////////////////////////////////////////////////////////////////
// file name : JTAGLib.h
// summary   : header file of class for JTAG BLAZER
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CJBLIB_H__E03793E7_8DD5_45CE_A48D_84ECB7F0143D__INCLUDED_)
#define AFX_CJBLIB_H__E03793E7_8DD5_45CE_A48D_84ECB7F0143D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vector>
#include <string>

#define MAX_FILE_NAME_LEN		256
#define MAX_VERSION_NAME_LEN	1024

using namespace std;

struct FlashFileData
{
	bool littleBear;
	bool compressed;
	time_t recordTime;
	unsigned long fileSize;
	char fileName[MAX_FILE_NAME_LEN + 1];
};

struct DataHeader
{
	unsigned long dataID;
	unsigned long version;
	unsigned long sequence;
};

class CJBLib
{
public:
	enum ErrorCode
	{
		JTAG_NO_ERROR				= 0x00000000,

		// common
		JTAG_OPEN_FILE_FAILED		= 0xffff0001,
		JTAG_READ_FILE_FAILED		= 0xffff0002,
		JTAG_OPEN_SOCKET_FAILED		= 0xffff0011,
		JTAG_SELECT_SOCKET_FAILED	= 0xffff0012,
		JTAG_RECV_SOCKET_FAILED		= 0xffff0013,
		JTAG_SEND_SOCKET_FAILED		= 0xffff0014,
		JTAG_FILESIZE_FAILED		= 0xffff0021,
		JTAG_FILE_NAME_LEN_FAILED	= 0xffff0022,
		JTAG_INVALID_RESPONSE		= 0xffff0023,
		JTAG_VERSION_MISMATCH		= 0xffff0024,
		JTAG_VERSION_NAME_FAILED	= 0xffff0025,
	};

	enum DataID
	{
		INVALID_DATA_ID				= 0x00000000,
		NET_WRITE_REQ				= 0x00010001,
		NET_WRITE_RES				= 0x00010002,
		NET_WRITE_NOTIFY			= 0x00010003,
		FLASH_WRITE_REQ				= 0x00020001,
		FLASH_WRITE_RES				= 0x00020002,
		FLASH_WRITE_NOTIFY			= 0x00020003,
		DEVICE_LIST_REQ				= 0x00030001,
		DEVICE_LIST_RES				= 0x00030002,
		UPDATE_PROG_REQ				= 0x00040001,
		UPDATE_PROG_RES				= 0x00040002,
		UPDATE_PROG_NOTIFY			= 0x00040003,
		MEMORY_SIZE_REQ				= 0x00050001,
		MEMORY_SIZE_RES				= 0x00050002,
		GET_TRANS_RATE_REQ			= 0x00060001,
		GET_TRANS_RATE_RES			= 0x00060002,
		SET_TRANS_RATE_REQ			= 0x00070001,
		SET_TRANS_RATE_RES			= 0x00070002,
		FLASH_DATA_REQ				= 0x00080001,
		FLASH_DATA_RES				= 0x00080002,
		GET_FIRM_VERSION_REQ		= 0x00090001,
		GET_FIRM_VERSION_RES		= 0x00090002,
	};

	enum MaxSizeType
	{
		CONFIG_DATA_SIZE	= 0,
		UPDATE_PROGRAM_SIZE	= 1,
	};

	CJBLib(unsigned long connectIP = 0);
	virtual ~CJBLib();

	ErrorCode TargetWriteRequest(
				const char* fileName,
				bool littleBear,
				void (*ProgressNotify)(int status, int param)
				);
	ErrorCode FlashWriteRequest(
				const char* fileName,
				const FlashFileData* fileData,
				void (*ProgressNotify)(int status, int param)
				);
	ErrorCode UpdateProgramRequest(
				const char* fileName,
				void (*ProgressNotify)(int status, int param)
				);
	ErrorCode DeviceListRequest(
				vector<unsigned int>* deviceList
				);
	ErrorCode MaxUpdateProgramSizeRequest(
				unsigned long* maxSize
				);
	ErrorCode MaxConfigDataSizeRequest(
				unsigned long* maxSize
				);
	ErrorCode TransferRateRequest(
				unsigned long *transferRate
				);
	ErrorCode ChangeTransferRateRequest(
				unsigned long transferRate
				);
	ErrorCode FlashFileDataRequest(
				FlashFileData* fileData
				);
	ErrorCode FirmwareVersionRequest(
				char* versionName
				);
	bool PrepareSocket(unsigned long targetIP = 0, unsigned short portNo = 0);
	bool CancelRequest();
	bool IsConnected()
		{ return (m_socket < 0)? false : true; };
	unsigned long GetConnectedIP() const
		{ return m_targetIP; };
	void CloseSocket();

private:
	int m_socket;
	unsigned int	m_sequence;
	unsigned short	m_portNo;
	unsigned long	m_targetIP;

protected:
	ErrorCode GetMaxDataSize(
				MaxSizeType type,
				unsigned long* maxSize
				);
	ErrorCode SendFlashFileData(
				const FlashFileData* fileData
				);
	ErrorCode RecvFlashFileData(
				FlashFileData* fileData
				);
	ErrorCode SendFileData(
				const char* fileName,
				DataID waitID,
				DataID notifyID,
				void (*NotifyFunc)(int status, int param)
	);
	unsigned int CalculateCheckSum(
				unsigned char* data,
				unsigned int size
				) const;
	bool SendHeader(
				DataID id
				);
	bool ReceiveHeader(
				DataHeader* header
				);
	ErrorCode ReceiveResponse(
				DataID id
				);
	bool SendLongData(
				unsigned long sendData
				);
	bool ReceiveLongData(
				unsigned long* recvData
				);
	unsigned long GetNewSequenceNo();
};

#endif // !defined(AFX_CJBLIB_H__E03793E7_8DD5_45CE_A48D_84ECB7F0143D__INCLUDED_)
